from rivr_rest.deserialization.utils import relations


def deserialize_json(resource):
    if isinstance(resource, (list, tuple)):
        return map(deserialize_json, resource)

    document = resource.get_attributes()

    if 'url' not in document:
        document['url'] = resource.get_uri()

    embedded, links = relations(resource)

    embed = lambda relation: (relation[0], deserialize_json(relation[1]))
    document.update(dict(map(embed, embedded)))

    link = lambda relation: (relation[0] + '_url', relation[1].get_uri())
    document.update(dict(map(link, links)))

    return document




import unittest
from rivr_rest.deserialization.json import deserialize_json
from resources import EmbeddedResource, TestResource


class JSONDeserializationTests(unittest.TestCase):
    def setUp(self):
        self.resource = TestResource()

    def test_includes_attributes(self):
        document = deserialize_json(self.resource)
        self.assertEqual(document['test'], 'value')

    def test_adds_url_for_self_to_document(self):
        document = deserialize_json(self.resource)
        self.assertEqual(document['url'], '/test')

    def test_embeds_resource(self):
        document = deserialize_json(self.resource)
        self.assertEqual(document['embed'], {
            'url': '/embed',
            'name': 'Embedded',
        })

    def test_embeds_collection_resource(self):
        document = deserialize_json(self.resource)
        self.assertEqual(document['collection'], [
            {
                'url': '/embed',
                'name': 'Embedded',
            }, {
                'url': '/embed',
                'name': 'Embedded',
            }
        ])

    def test_links_resource(self):
        document = deserialize_json(self.resource)
        self.assertEqual(document['link_url'], '/embed')



import unittest
from rivr.http import Request
from rivr_rest.resource import Resource


class TestResource(Resource):
    uri_template = '/resource/{id}'

    def get_parameters(self):
        return {
            'id': 5,
        }

    def get_relations(self):
        return {
            'next': TestResource(parameters={'id': 6})
        }


class ResourceTests(unittest.TestCase):
    def setUp(self):
        self.resource = TestResource()

    # URI

    def test_uri_template(self):
        self.assertEqual(self.resource.uri_template, '/resource/{id}')

    def test_uri_expansion(self):
        self.assertEqual(self.resource.get_uri(), '/resource/5')

    # Parameters

    def test_parameters(self):
        resource = Resource(parameters={'id': 6})
        self.assertEqual(resource.get_parameters(), {'id': 6})

    # Can Embed

    def test_cannot_embed_self(self):
        self.assertFalse(self.resource.can_embed('self'))

    def test_cannot_embed_next(self):
        self.assertFalse(self.resource.can_embed('next'))

    def test_cannot_embed_prev(self):
        self.assertFalse(self.resource.can_embed('prev'))

    def test_can_embed_arbitary_relation(self):
        self.assertTrue(self.resource.can_embed('arbitary'))

    #

    def test_content_types(self):
        self.assertEqual(self.resource.content_type_providers().keys(),
            ['application/json', 'application/hal+json'])

    def test_preferred_content_type(self):
        self.assertEqual(self.resource.preferred_content_type,
            'application/json')

    def test_get(self):
        request = Request()
        response = self.resource.get(request)

        self.assertEqual(response.headers['Content-Type'],
            'application/json; charset=utf8')

        self.assertEqual(response.content,
            '{"url": "/resource/5", "next_url": "/resource/5"}')




import re
from rivr.http import Http404
from rivr_rest.resource import Resource


EXTRACT_VARIABLE_REGEX = re.compile(r'\{([\w]+)\}')

def extract(uri_template, uri):
    """
    Reverse URI Template implementation.

    Note, only simple variable templates are currently supported.
    """

    if uri == uri_template:
        return {}

    escaped_uri_template = re.escape(uri_template).replace('\{', '{').replace('\}', '}')

    def replace(match):
        return '(?P<{}>[\w]+)'.format(match.group(1))

    pattern = '^{}$'.format(
        re.sub(EXTRACT_VARIABLE_REGEX, replace, escaped_uri_template))

    match = re.match(pattern, uri)
    if match:
        return match.groupdict()

    return None


class RootResource(Resource):
    uri_template = '/'

    def __init__(self, resources):
        self.resources = resources

    def get_relations(self):
        def to_resource(relation):
            if isinstance(relation[1], type):
                return (relation[0], relation[1]())

            return relation

        return dict(map(to_resource, self.resources.items()))

    def can_embed(self, relation):
        return False


class Router(object):
    root_resource = RootResource

    def __init__(self, *resources):
        self.root_resources = {}
        self.resources = list(resources)

    def register(self, resource):
        self.resources.append(resource)

    def add_root_resource(self, relation, resource):
        self.root_resources[relation] = resource

    def resolve(self, path):
        for resource in self.resources:
            parameters = extract(resource.uri_template, path)
            if parameters is not None:
                return (resource, parameters)

        return None

    def __call__(self, request):
        if request.path == '/' and len(self.root_resources) > 0:
            resource = self.root_resource(resources=self.root_resources)
            return resource.dispatch(request)

        match = self.resolve(request.path)
        if not match:
            raise Http404

        resource, parameters = match

        return resource(parameters=parameters).dispatch(request)




from rivr_rest.resource import Resource


class EmbeddedResource(Resource):
    uri_template = '/embed'

    def get_attributes(self):
        return { 'name': 'Embedded' }

class TestResource(Resource):
    uri_template = '/test'

    def get_attributes(self):
        return { 'test': 'value' }

    def get_relations(self):
        return {
            'embed': EmbeddedResource(),
            'collection': [EmbeddedResource(), EmbeddedResource()],
            'link': EmbeddedResource(),
        }

    def can_embed(self, relation):
        return relation is not 'link'





def partition(function, iterable):
    trues = []
    falses = []

    for item in iterable:
        if function(item):
            trues.append(item)
        else:
            falses.append(item)

    return (trues, falses)


def relations(resource):
    return partition(lambda relation: resource.can_embed(relation[0]),
                     resource.get_relations().items())





import unittest
from rivr_rest import Resource, Router
from rivr_rest.router import extract


class RootResource(Resource):
    uri_template = '/'


class UsersResource(Resource):
    uri_template = '/users'


class RouterTests(unittest.TestCase):
    def test_register_resource(self):
        router = Router()
        router.register(RootResource)

        self.assertEqual(len(router.resources), 1)

    def test_routing_path(self):
        router = Router(RootResource, UsersResource)
        root, root_params = router.resolve('/')
        users, users_params = router.resolve('/users')

        self.assertEqual(root, RootResource)
        self.assertEqual(users, UsersResource)
        self.assertIsNone(router.resolve('/unknown'))


class URITemplateExtractTests(unittest.TestCase):
    def test_no_variables_not_match(self):
        self.assertIsNone(extract('/pathy', '/path'))

    def test_no_variables(self):
        self.assertEqual(extract('/path', '/path'), {})

    def test_simple_variable(self):
        self.assertEqual(extract('/{name}', '/kyle'), {'name': 'kyle'})

    def test_simple_variables(self):
        self.assertEqual(extract('/{x}/{y}', '/a/bcd'), {'x': 'a', 'y': 'bcd'})





import unittest
from rivr_rest.deserialization.hal import deserialize_hal
from resources import EmbeddedResource, TestResource


class HALDeserializationTests(unittest.TestCase):
    def setUp(self):
        self.resource = TestResource()

    def test_includes_attributes(self):
        document = deserialize_hal(self.resource)
        self.assertEqual(document['test'], 'value')

    def test_adds_link_for_self_to_document(self):
        document = deserialize_hal(self.resource)
        self.assertEqual(document['_links']['self'], {'href':'/test'})

    def test_links_resource(self):
        document = deserialize_hal(self.resource)
        self.assertEqual(document['_links']['link'], {'href':'/embed'})

    def test_embeds_resource(self):
        document = deserialize_hal(self.resource)
        self.assertEqual(document['_embedded']['embed'], {
            '_links': {'self': {'href': '/embed'}},
            'name': 'Embedded'
        })

    def test_embeds_collection_resource(self):
        document = deserialize_hal(self.resource)
        self.assertEqual(document['_embedded']['collection'], [
            {
                '_links': {'self': {'href': '/embed'}},
                'name': 'Embedded'
            }, {
                '_links': {'self': {'href': '/embed'}},
                'name': 'Embedded',
            }
        ])



import json

from uritemplate import expand
from negotiator import AcceptParameters, ContentType, ContentNegotiator

from rivr.views import View
from rivr.http import Response
from rivr_rest.deserialization.json import deserialize_json
from rivr_rest.deserialization.hal import deserialize_hal


class Resource(View):
    uri_template = None
    parameters = None

    def __init__(self, parameters=None):
        self.parameters = parameters

    def get_uri(self):
        """
        Returns the URI for this resource, using the parameters for expansion.
        """
        parameters = self.get_parameters()
        return expand(self.uri_template, parameters)

    ###

    def get_parameters(self):
        """
        Returns the parameters used for the resource.
        """
        return self.parameters or {}

    def get_attributes(self):
        return {}

    def get_relations(self):
        """
        Returns a dictionary of relations, key must be a string and the value
        should be another resource or a list of resources.
        """
        return {}

    def can_embed(self, relation):
        """
        When this method returns `True` for a given relation, the relation will
        be embedded in any resource when applicable.

        By default, this returns True unless the relation is `self`,
        `next`, or `prev`.
        """
        return relation not in ('self', 'next', 'prev')

    ###

    preferred_content_type = 'application/json'

    def content_type_providers(self):
        """
        Returns a dictionary of content type providers, key is a string
        of the content type, i.e, `application/json` and the value is a
        function that returns a response for that type.
        """

        def json_provider(deserializer, content_type):
            def inner():
                content = json.dumps(deserializer(self))
                return Response(content, content_type='{}; charset=utf8'.format(content_type))
            return inner

        return {
            'application/json': json_provider(deserialize_json, 'application/json'),
            'application/hal+json': json_provider(deserialize_hal, 'application/hal+json'),
        }

    def get(self, request):
        content_type_to_acceptable = lambda content_type: AcceptParameters(ContentType(content_type))
        acceptable = map(content_type_to_acceptable, self.content_type_providers().keys())

        accept = request.headers.get('HTTP_ACCEPT')
        negotiator = ContentNegotiator(
            content_type_to_acceptable(self.preferred_content_type),
            acceptable)
        negotiated_type = negotiator.negotiate(accept=accept)
        content_type = negotiated_type.content_type or self.preferred_content_type

        provider = self.content_type_providers()[str(content_type)]

        return provider()




#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='rivr-rest',
    version='0.1.0',
    description='Library for building REST apis.',
    url='https://github.com/rivrproject/rivr-rest',
    packages=find_packages(),
    install_requires=[
        'rivr',
        'uritemplate',
        'negotiator',
    ],
    author='Kyle Fuller',
    author_email='inbox@kylefuller.co.uk',
    license='BSD',
    classifiers=(
      'Programming Language :: Python :: 2',
      'Programming Language :: Python :: 2.7',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.2',
      'Programming Language :: Python :: 3.3',
      'Programming Language :: Python :: 3.4',
      'License :: OSI Approved :: BSD License',
    )
)




from rivr_rest.deserialization.utils import relations


def deserialize_hal(resource):
    if isinstance(resource, (list, tuple)):
        return map(deserialize_hal, resource)

    embedded, links = relations(resource)
    embed = lambda relation: (relation[0], deserialize_hal(relation[1]))

    link = lambda relation: (relation[0], {'href':relation[1].get_uri()})
    links = dict(map(link, links))
    links['self'] = { 'href': resource.get_uri() }

    document = resource.get_attributes()
    document['_links'] = links
    if len(embedded) > 0:
        document['_embedded'] = dict(map(embed, embedded))

    return document

